require 'mongoid'
require 'oslc'
require 'mongoid/oslc/config'
require 'mongoid/oslc/version'
require 'mongoid/oslc/exceptions'
require 'mongoid/oslc/strategy'
require 'mongoid/oslc/resources'
require 'active_support/concern'

module Mongoid
  module Oslc
    extend ActiveSupport::Concern

    @@resources = Resources.new

    def self.resources
      @@resources
    end

    def domain
      Mongoid::Oslc.resources[self.class.name][:domain]
    end

    def describe
      Mongoid::Oslc.resources[self.class.name][:describe]
    end

    def oslc_value(oslc_name)
      field_name = Mongoid::Oslc.resources.field_name(oslc_name)
      self[field_name]
    end

    module ClassMethods
      def domain(name)
        Mongoid::Oslc.resources[self.name].merge!(domain: name)
      end

      def describe(describe)
        Mongoid::Oslc.resources[self.name].merge!(describe: describe)
      end

      def oslc_field(name, options = {})
        oslc_options = options_for_oslc(options)
        field(name, options)
        register_oslc_field(name, oslc_options)
      end

      def oslc_embeds_many(name, options = {})
        oslc_options = options_for_oslc(options)
        embeds_many(name, options)
        embedded_class = relations[name.to_s].class_name.constantize
        register_oslc_field(name, oslc_options.merge(type: Array, embedded: embedded_class))
      end

      def oslc_fields
        (ancestors - included_modules).inject({}) do |hash, ancestor_class|
          hash.merge(Mongoid::Oslc.resources[ancestor_class.name][:fields])
        end
      end

      def oslc_where(query)
        mongoid_query = Strategy.to_query(query)
        where(mongoid_query)
      end

      def register_oslc_field(name, options)
        Mongoid::Oslc.resources[self.name][:fields][options[:mapping]] = {
          name: name,
          value_type: ::Oslc::VALUE_TYPES[options[:value_type] || fields[name.to_s].type.to_s],
          read_only: (options[:read_only] || false),
          occurs: occurs(options),
          allowed_values: (options[:allowed_values] || []),
          embedded: (options[:embedded] || false),
          range: (options[:range] || [])
        }
      end

      private

      def occurs(options)
        mandatory = options[:mandatory] || false
        case
          when mandatory && options[:type] == Array then ::Oslc::ONE_OR_MANY
          when mandatory                            then ::Oslc::EXACTLY_ONE
          when options[:type] == Array              then ::Oslc::ZERO_OR_MANY
          else ::Oslc::ZERO_OR_ONE
        end
      end

      def options_for_oslc(options)
        [:value_type, :read_only, :allowed_values, :mapping, :mandatory, :range].inject({}) do |hash, field|
          hash.merge(field => options.delete(field))
        end.reject{|field, value| value.nil?}.merge(type: options[:type])
      end
    end
  end
end
