require 'treetop'

module Mongoid
  module Oslc
    module Grammar
      class BinaryDecision < Treetop::Runtime::SyntaxNode
        def to_mongo_query
          operand_1.to_mongo_query.deep_merge(operand_2.to_mongo_query)
        end
      end

      class Operator < Treetop::Runtime::SyntaxNode
        OPERATORS = {
          "="   => "$in",
          "!="  => "$ne",
          ">"   => "$gt",
          ">="  => "$gte",
          "<"   => "$lt",
          "<="  => "$lte",
          "in"  => "$in"
        }

        def to_mongo_query_with_value(value)
          if text_value == "="
            value
          else
            { OPERATORS[text_value] => value }
          end
        end
      end

      class InOp < Treetop::Runtime::SyntaxNode
        def to_mongo_query_with_value(value)
          { "$in" => value }
        end
      end

      class InVal < Treetop::Runtime::SyntaxNode
        def to_mongo_query
          [first_value.to_mongo_query] + rest_values.elements.map{|comma_and_value| comma_and_value.rest_value.to_mongo_query }
        end
      end

      class UriRefEsc < Treetop::Runtime::SyntaxNode
        def to_mongo_query
          # TODO: an angle bracket-delimited URI reference in which > and \ are \-escaped.
          text_content
        end
      end

      class BooleanValue < Treetop::Runtime::SyntaxNode
        def to_mongo_query
          text_value == "true"
        end
      end

      class BlankValue < Treetop::Runtime::SyntaxNode
        def to_mongo_query
          nil
        end
      end

      class DecimalValue < Treetop::Runtime::SyntaxNode
        def to_mongo_query
          text_value.to_f
        end
      end

      class StringEsc < Treetop::Runtime::SyntaxNode
        def to_mongo_query
          value = text_value[1...-1]
          if value.starts_with?("*") || value.ends_with?("*")
            value.starts_with?("*") ? value.slice!(0)  : value.prepend("\\A")
            value.ends_with?("*")   ? value.slice!(-1) : value.concat("\\z")
            value = Regexp.new(value)
          end
          value
        end
      end

      class Condition < Treetop::Runtime::SyntaxNode
        def to_mongo_query
          oslc_name = field_name.text_value
          field     = Mongoid::Oslc.resources.field_name(oslc_name)
          value     = field_value.to_mongo_query
          value     = [] if value.nil? && Mongoid::Oslc.resources.is_field_array?(oslc_name)
          { field => operator.to_mongo_query_with_value(value) }
        end
      end
    end
  end
end
